﻿// Copyright (c) 2010-2013 SharpDX - Alexandre Mutel
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
// -----------------------------------------------------------------------------
// Original code from SlimDX project.
// Greetings to SlimDX Group. Original code published with the following license:
// -----------------------------------------------------------------------------
/*
* Copyright (c) 2007-2009 SlimDX Group
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*/
using System;
using System.Runtime.InteropServices;
using SharpDX;
using SharpDX.Direct2D1;
using SharpDX.Direct3D10;
using SharpDX.Direct3D9;
using SharpDX.Windows;
using Color = SharpDX.Color;
using D2DFactory = SharpDX.Direct2D1.Factory;
using Device1 = SharpDX.Direct3D10.Device1;
using DWriteFactory = SharpDX.DirectWrite.Factory;
using FeatureLevel = SharpDX.Direct3D10.FeatureLevel;
using PresentFlags = SharpDX.Direct3D9.PresentFlags;
using RectangleF = SharpDX.RectangleF;
using Surface = SharpDX.Direct3D9.Surface;
using Usage = SharpDX.Direct3D9.Usage;
namespace MiniTri
{
  [StructLayout(LayoutKind.Sequential)]
  struct Vertex
  {
    public Vector4 Position;
    public ColorBGRA Color;
  }

  /// <summary>
  ///   SlimDX2 port of SlimDX-MiniTri Direct3D9 Sample
  /// </summary>
  internal static class Program
  {
    [STAThread]
    private static void Main()
    {
      var form = new RenderForm("SlimDX2 - MiniTri Direct3D9 Sample");
      int width = form.ClientSize.Width;
      int height = form.ClientSize.Height;
      var presentParameters = new PresentParameters(width, height)
      {
        SwapEffect = SwapEffect.FlipEx,
        PresentFlags = PresentFlags.None,
        MultiSampleType = MultisampleType.None,
        MultiSampleQuality = 0
      };
      var deviceEx = new DeviceEx(new Direct3DEx(), 0, DeviceType.Hardware, form.Handle,
        CreateFlags.HardwareVertexProcessing | CreateFlags.Multithreaded, presentParameters);
      var device = new Device1(DriverType.Hardware, DeviceCreationFlags.BgraSupport, FeatureLevel.Level_10_1);

      var vertices = new VertexBuffer(deviceEx, 3*20, Usage.WriteOnly, VertexFormat.None, Pool.Default);
      vertices.Lock(0, 0, LockFlags.None).WriteRange(new[]
      {
        new Vertex {Color = Color.Red, Position = new Vector4(400.0f, 100.0f, 0.5f, 1.0f)},
        new Vertex {Color = Color.Blue, Position = new Vector4(650.0f, 500.0f, 0.5f, 1.0f)},
        new Vertex {Color = Color.Green, Position = new Vector4(150.0f, 500.0f, 0.5f, 1.0f)}
      });
      vertices.Unlock();

      var vertexElems = new[]
      {
        new VertexElement(0, 0, DeclarationType.Float4, DeclarationMethod.Default, DeclarationUsage.PositionTransformed,
          0),
        new VertexElement(0, 16, DeclarationType.Color, DeclarationMethod.Default, DeclarationUsage.Color, 0),
        VertexElement.VertexDeclarationEnd
      };

      var vertexDecl = new VertexDeclaration(deviceEx, vertexElems);


      SharedTextureContainer sharedTexture = new SharedTextureContainer(deviceEx, device, width, height);
      sharedTexture.Allocate();

      var rectangleGeometry = new RoundedRectangleGeometry(sharedTexture.D2DFactory,
        new RoundedRectangle
        {
          RadiusX = 32,
          RadiusY = 32,
          Rect = new RectangleF(128, 128, width - 128*2, height - 128*2)
        });
      var solidColorBrush = new SolidColorBrush(sharedTexture.RenderTarget2D, Color.White);

      Surface backBuffer = deviceEx.GetRenderTarget(0);

      RenderLoop.Run(form, () =>
      {
        //sharedTexture.Render9(dev => dev.Clear(ClearFlags.Target | ClearFlags.ZBuffer, Color.Black, 1.0f, 0));

        sharedTexture.Render9(dev =>
        {
          dev.Clear(ClearFlags.Target | ClearFlags.ZBuffer, Color.Black, 1.0f, 0);
          dev.BeginScene();

          dev.SetStreamSource(0, vertices, 0, 20);
          dev.VertexDeclaration = vertexDecl;
          dev.DrawPrimitives(PrimitiveType.TriangleList, 0, 1);

          dev.EndScene();
        });


        sharedTexture.Render10(target =>
        {
          target.BeginDraw();
          solidColorBrush.Color = new Color4(0xFF60A0C0);
          target.FillGeometry(rectangleGeometry, solidColorBrush, null);
          target.EndDraw();

        });

        deviceEx.StretchRectangle(sharedTexture.SharedSurface9, backBuffer, TextureFilter.None);
        deviceEx.Present();
      });

      sharedTexture.Dispose();
    }

    //public static RenderTarget RenderTarget2D { get; set; }
  }
}